@@ -16,6 +16,7 @@
 
 import asyncio
 import bisect
+from datetime import datetime
 from hashlib import sha256
 from typing import TYPE_CHECKING, Any, MutableMapping, MutableSequence, Optional, Tuple
 
@@ -67,6 +68,11 @@ def _get_event_data(event: Any) -> MutableMapping[str, Any]:
     return {}
 
 
+def _unpack_args(args: Tuple[Any, ...]) -> str:
+    """Unpack arguments into a string for logging purposes."""
+    return ", ".join([str(arg) for arg in args])
+
+
 class EventDispatcher(MixinBase):
     # Initialized during instantiation
     listeners: MutableMapping[str, MutableSequence[Listener]]
@@ -189,6 +195,9 @@ class EventDispatcher(MixinBase):
                 dispatcher_error = EventDispatchError(
                     f"raised from {type(err).__name__}: {str(err)}"
                 ).with_traceback(err.__traceback__)
+                await self.dispatch_alert(
+                    f"Event __{event}__ on `{lst.func.__qualname__}`", dispatcher_error
+                )
                 if is_tg_event and args[0] is not None:
                     data = _get_event_data(args[0])
                     self.log.error(
@@ -211,7 +220,7 @@ class EventDispatcher(MixinBase):
                         "Error dispatching event '%s' on %s with data\n%s",
                         event,
                         lst.func.__qualname__,
-                        args,
+                        _unpack_args(args),
                         exc_info=dispatcher_error,
                     )
                 continue
@@ -292,8 +301,12 @@ class EventDispatcher(MixinBase):
 
                     await asyncio.wait(
                         (
-                            send_missed_message(diff.new_messages, users, chats),
-                            send_missed_update(diff.other_updates, users, chats),
+                            self.loop.create_task(
+                                send_missed_message(diff.new_messages, users, chats)
+                            ),
+                            self.loop.create_task(
+                                send_missed_update(diff.other_updates, users, chats)
+                            ),
                         )
                     )
                 elif isinstance(diff, raw.types.updates.difference_empty.DifferenceEmpty):
@@ -315,5 +328,32 @@ class EventDispatcher(MixinBase):
                 {"$unset": {"pts": "", "date": "", "qts": "", "seq": ""}},
             )
 
+    async def dispatch_alert(self: "Anjani", invoker: str, exc: BaseException) -> None:
+        """Dispatches an alert to the configured alert log."""
+        if not self.config["alert_log"]:
+            return
+
+        log_chat = self.config["alert_log"].split("#")
+        thread_id = None
+        chat_id = int(log_chat[0])
+        if len(log_chat) == 2:
+            thread_id = int(log_chat[1])
+
+        alert = f"""🔴 **Anjani ERROR ALERT**
+
+  - **Alert by:** {invoker}
+  - **Time (UTC):** {datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")}
+
+**ERROR**
+```python
+{util.error.format_exception(exc)}
+```
+        """
+        await self.client.send_message(
+            chat_id,
+            alert,
+            message_thread_id=thread_id,  # type: ignore
+        )
+
     async def log_stat(self: "Anjani", stat: str, *, value: int = 1) -> None:
         await self.dispatch_event("stat_listen", stat, value)
