@@ -37,6 +37,7 @@ from pyrogram.errors import (
     ChatAdminRequired,
     Forbidden,
     PeerIdInvalid,
+    UserAdminInvalid,
 )
 from pyrogram.types import (
     CallbackQuery,
@@ -59,6 +60,7 @@ class Federation(plugin.Plugin):
 
     async def on_load(self) -> None:
         self.db = self.bot.db.get_collection("FEDERATIONS")
+        self.chat_db = self.bot.db.get_collection("CHATS")
 
     async def on_chat_migrate(self, message: Message) -> None:
         new_chat = message.chat.id
@@ -113,7 +115,12 @@ class Federation(plugin.Plugin):
                 self.text(chat.id, "fed-autoleave", fed_data["name"], fed_data["_id"]),
                 self.db.update_one({"_id": fed_data["_id"]}, {"$pull": {"chats": chat.id}}),
             )
-            await self.bot.client.send_message(chat.id, ret)
+            thread_id = await self.get_action_topic(chat.id)
+            await self.bot.client.send_message(
+                chat.id,
+                ret,
+                message_thread_id=thread_id,  # type: ignore
+            )
 
     @listener.filters(filters.regex(r"(rm|log)fed_(.*)"))
     async def on_callback_query(self, query: CallbackQuery) -> Any:
@@ -161,6 +168,10 @@ class Federation(plugin.Plugin):
         if banned:
             await self.fban_handler(chat, target, banned)
 
+    async def get_action_topic(self, chat_id: int) -> Optional[int]:
+        data = await self.chat_db.find_one({"chat_id": chat_id}, {"action_topic": 1})
+        return data.get("action_topic") if data else None
+
     @staticmethod
     def is_fed_admin(data: Mapping[str, Any], user: int) -> bool:
         """Check federation admin"""
@@ -328,11 +339,14 @@ class Federation(plugin.Plugin):
                         data["reason"],
                         data["time"].strftime("%Y %b %d %H:%M UTC"),
                     ),
+                    message_thread_id=await self.get_action_topic(chat.id),  # type: ignore
                 ),
                 self.bot.client.ban_chat_member(chat.id, user.id),
             )
-        except ChatAdminRequired:
-            self.log.debug(f"Can't ban user {user.username} on {chat.title}")
+        except (ChatAdminRequired, UserAdminInvalid) as e:
+            self.log.warning(
+                f"Can't ban user {user.username} on {chat.title} ({chat.id}) caused by {e}"
+            )
 
     async def cmd_newfed(self, ctx: command.Context, name: Optional[str] = None) -> str:
         """Create a new federations"""
@@ -677,6 +691,34 @@ class Federation(plugin.Plugin):
             reason,
         )
 
+    async def _propagate_fban(
+        self,
+        target: Union[User, Chat],
+        chats: List[int],
+        host_fed: str,
+        sub_fed: Optional[str] = None,
+    ) -> Mapping[int, str]:
+        """Propagate fban to each chat in the list"""
+        failed = {}
+        for chat in chats:
+            try:
+                await self.bot.client.ban_chat_member(chat, target.id)
+            except BadRequest as br:
+                self.log.warning(
+                    f"Failed to send fban on subfed {sub_fed} of {host_fed} at {chat} due to {br.MESSAGE}"
+                    if sub_fed
+                    else f"Failed to fban {target.username} on {chat} due to {br.MESSAGE}"
+                )
+                failed[chat] = br.MESSAGE
+            except (Forbidden, ChannelPrivate) as err:
+                self.log.warning(
+                    f"Can't fban on subfed {sub_fed} of {host_fed} at {chat} caused by {err.MESSAGE}"
+                    if sub_fed
+                    else f"Can't fban {target.username} on {chat} caused by {err.MESSAGE}"
+                )
+                failed[chat] = err.MESSAGE
+        return failed
+
     async def cmd_fban(
         self, ctx: command.Context, target: Union[User, Chat, None] = None, *, reason: str = ""
     ) -> Optional[str]:
@@ -728,18 +770,8 @@ class Federation(plugin.Plugin):
         else:
             return await self.text(chat.id, "err-peer-invalid")
 
-        failed: Dict[int, Optional[str]] = {}
-        for chat in data["chats"]:
-            try:
-                await self.bot.client.ban_chat_member(chat, target.id)
-            except BadRequest as br:
-                self.log.warning(f"Failed to fban {target.username} on {chat} due to {br.MESSAGE}")
-                failed[chat] = br.MESSAGE
-            except (Forbidden, ChannelPrivate) as err:
-                self.log.warning(
-                    f"Can't to fban {target.username} on {chat} caused by {err.MESSAGE}"
-                )
-                failed[chat] = err.MESSAGE
+        await ctx.respond(f"Starting a federation ban for {target.id} in federation {data['name']}")
+        failed = await self._propagate_fban(target, data["chats"], data["_id"])
 
         text = ""
         if failed:
@@ -755,17 +787,9 @@ class Federation(plugin.Plugin):
                 subs_data = await self.get_fed(fed_id)
                 if not subs_data:
                     continue
-                for chat in subs_data.get("chats", []):
-                    try:
-                        await self.bot.client.ban_chat_member(chat, target.id)
-                    except BadRequest as br:
-                        self.log.warning(
-                            f"Failed to send fban on subfed {subs_data['_id']} of {data['_id']} at {chat} due to {br.MESSAGE}"
-                        )
-                    except (Forbidden, ChannelPrivate) as err:
-                        self.log.warning(
-                            f"Can't to fban on subfed {subs_data['_id']} of {data['_id']} at {chat} caused by {err.MESSAGE}"
-                        )
+                await self._propagate_fban(
+                    target, subs_data.get("chats", []), data["_id"], subs_data["_id"]
+                )
 
         await ctx.respond(string)
 
@@ -777,6 +801,24 @@ class Federation(plugin.Plugin):
 
         return None
 
+    async def _propagate_unfban(
+        self,
+        target: Union[User, Chat],
+        chats: List[int],
+        host_fed: str,
+        sub_fed: Optional[str] = None,
+    ) -> None:
+        for chat in chats:
+            try:
+                await self.bot.client.unban_chat_member(chat, target.id)
+                await asyncio.sleep(0.5)
+            except (BadRequest, Forbidden, ChannelPrivate) as err:
+                self.log.warning(
+                    f"Failed to unfban on subfed {sub_fed} of {host_fed} due to {err.MESSAGE}"
+                    if sub_fed
+                    else f"Failed to unfban on {host_fed} due to {err.MESSAGE}"
+                )
+
     async def cmd_unfban(self, ctx: command.Context, target: Union[User, Chat, None] = None) -> str:
         """Unban a user on federation"""
         chat = ctx.chat
@@ -830,24 +872,18 @@ class Federation(plugin.Plugin):
         else:
             return ""
 
-        for chat in data["chats"]:
-            try:
-                await self.bot.client.unban_chat_member(chat, target.id)
-            except (BadRequest, Forbidden, ChannelPrivate) as err:
-                self.log.warning(f"Failed to unfban on {data['_id']} due to {err.MESSAGE}")
+        await ctx.respond(f"Removing federation ban for {target.id} in federation {data['name']}")
+        await self._propagate_unfban(target, data["chats"], data["_id"])
 
         if data.get("subscribers", []):
             for fed_id in data["subscribers"]:
                 subs_data = await self.get_fed(fed_id)
                 if not subs_data:
                     continue
-                for chat in subs_data.get("chats", []):
-                    try:
-                        await self.bot.client.unban_chat_member(chat, target.id)
-                    except (BadRequest, Forbidden, ChannelPrivate) as err:
-                        self.log.warning(
-                            f"Failed to unfban on subfed {subs_data['_id']} of {data['_id']} due to {err.MESSAGE}"
-                        )
+
+                await self._propagate_unfban(
+                    target, subs_data.get("chats", []), data["_id"], subs_data["_id"]
+                )
 
         if log := data.get("log"):
             await self.bot.client.send_message(log, text, disable_web_page_preview=True)
@@ -861,7 +897,7 @@ class Federation(plugin.Plugin):
         if len(ctx.args) > 1:  # <user_id> <fed_id>
             try:
                 user_id = int(ctx.args[0])
-            except TypeError:
+            except (TypeError, ValueError):
                 return await self.text(chat.id, "fed-invalid-user-id")
 
             data = await self.get_fed(ctx.args[1])
@@ -904,7 +940,7 @@ class Federation(plugin.Plugin):
                     user = user[0]
                 if not user:
                     return await self.text(chat.id, "fed-invalid-user-id")
-            except TypeError:
+            except (TypeError, ValueError):
                 return await self.text(chat.id, "fed-invalid-user-id")
 
         reply_msg = ctx.msg.reply_to_message
